//#ifndef __CINT__
#include "RooGlobalFunc.h"
//#endif
#include "RooRealVar.h"
#include "RooArgList.h"
#include "RooFormulaVar.h"
#include "RooDataSet.h"
#include "RooGaussian.h"
#include "RooHist.h"
#include "RooCBShape.h"
#include "RooPolynomial.h"
#include "RooBinning.h"
#include "RooPolyVar.h"
#include "TH1.h"
#include "TH2.h"
#include "RooAddPdf.h"
#include "RooProdPdf.h"
#include "RooFitResult.h"
#include "RooGenericPdf.h"
#include "RooLandau.h"
#include "TChain.h"
#include "TCanvas.h"
#include "TAxis.h"
#include "TLegend.h"
#include "RooPlot.h"
#include "RooCategory.h"
#include "RooSuperCategory.h"
#include "RooSimultaneous.h"
#include "RooNLLVar.h"
#include "TFile.h"
#include "RooJohnson.h"
#include "RooKeysPdf.h"
#include "TRandom.h"
#include "TPaveLabel.h"
#include "TLegend.h"
#include "TStyle.h"
#include "TLatex.h"
#include "TROOT.h"
#include "/home/sanjeeda/Desktop/thesis_new/analysis/D0_KSKS/2d_fit/gen_MC/complete_fit/myRooPolBG.cpp"

using namespace RooFit ;
using namespace std;

const int    belle2_font = 133;
const double belle2_tsize = 26;

void set_style()
{
 
  gROOT->SetStyle("Modern");
  TStyle *belle2Style = new TStyle("BelleII","Belle II style");
  
  // set the paper & margin sizes
  belle2Style->SetPaperSize(20,26);
  
  // set canvas, pads, etc.
  belle2Style->SetCanvasDefW(600);
  belle2Style->SetCanvasDefH(600);
  
  belle2Style->SetFillColor(1);
  belle2Style->SetFillStyle(1001);
  belle2Style->SetFrameFillColor(0);
  belle2Style->SetFrameBorderMode(0);
  belle2Style->SetPadBorderMode(0);
  belle2Style->SetPadColor(0);
  belle2Style->SetCanvasBorderMode(0);
  belle2Style->SetCanvasColor(0);
  belle2Style->SetStatColor(0);
  belle2Style->SetLegendBorderSize(0);
  
  // set margin sizes
  belle2Style->SetPadTopMargin(0.05);
  belle2Style->SetPadRightMargin(0.05);
  belle2Style->SetPadBottomMargin(0.16);
  belle2Style->SetPadLeftMargin(0.16);
  
  // use large fonts
  belle2Style->SetTextFont(belle2_font);
  belle2Style->SetTextSize(belle2_tsize);
  
  belle2Style->SetLabelFont(belle2_font,"x");
  belle2Style->SetTitleFont(belle2_font,"x");
  belle2Style->SetLabelFont(belle2_font,"y");
  belle2Style->SetTitleFont(belle2_font,"y");
  belle2Style->SetLabelFont(belle2_font,"z");
  belle2Style->SetTitleFont(belle2_font,"z");
  
  belle2Style->SetLabelSize(belle2_tsize,"x");
  belle2Style->SetLabelSize(belle2_tsize,"y");
  belle2Style->SetLabelSize(belle2_tsize,"z");
  belle2Style->SetTitleSize(belle2_tsize,"x");
  belle2Style->SetTitleSize(belle2_tsize,"y");
  belle2Style->SetTitleSize(belle2_tsize,"z");
  
  belle2Style->SetTitleOffset(1.5,"x");
  belle2Style->SetTitleOffset(1.5,"y");
  belle2Style->SetTitleOffset(1.5,"z");
  
  belle2Style->SetLabelOffset(0.01,"x");
  belle2Style->SetLabelOffset(0.01,"y");
  belle2Style->SetLabelOffset(0.01,"z");
  
  belle2Style->SetLegendFont(belle2_font);
   
  // use bold lines and markers
  belle2Style->SetMarkerStyle(20);
  belle2Style->SetMarkerSize(1.0);
  belle2Style->SetHistLineWidth(2);
  belle2Style->SetFrameLineWidth(2);
  belle2Style->SetLineWidth(2);
  belle2Style->SetFuncWidth(2);
  belle2Style->SetLineStyleString(2,"[12 12]");// postscript dashes
  
  // get rid of X error bars
  //belle2Style->SetErrorX(0.001);
   // get rid of error bar caps
  belle2Style->SetEndErrorSize(0.);
  
  // do not display any of the standard histogram decorations
  belle2Style->SetOptTitle(0);
  belle2Style->SetOptStat(0);
  belle2Style->SetOptFit(0);
  
  // put tick marks on top and RHS of plots
  belle2Style->SetPadTickX(1);
  belle2Style->SetPadTickY(1);
  
  // histogram divisions: only 5 in x to avoid label overlaps
  belle2Style->SetNdivisions(505,"x");
  belle2Style->SetNdivisions(510,"y");
  
  // function
  belle2Style->SetFuncColor(kBlue);
  belle2Style->SetFuncStyle(0);
  
  // legend
  belle2Style->SetLegendBorderSize(0);
  belle2Style->SetLegendFillColor(0);
  
  belle2Style->SetHistMinimumZero();
  
  std::cout << "-------------------------" << std::endl;
  std::cout << "Set Belle II Style" << std::endl;
  std::cout << "-------------------------" << std::endl;
   
  gROOT->SetStyle("BelleII");
  gROOT->ForceStyle();
  
  // TGaxis::SetMaxDigits(4);
}

void mc15_asymmetry_fit_b2style()
{
  TFile *f = new TFile("new_component_wise_trees_for_fit_data.root");

  TTree *tree = (TTree*)f->Get("complete");
  RooRealVar log_min_significance("log_min_significance","", -6, 7, "");
  // RooRealVar deltaM("deltaM","", 0.139, 0.159, "");
  RooRealVar d0pi("d0pi","", 2.005, 2.0245, "");

  //Variable used to separate the total sample into two (should be there in the tree)
  RooRealVar pisoft("pisoft","",-2,2,"");

  Double_t constant_value = 1.86484;
  RooRealVar d0_mass("d0_mass", "d0_mass", constant_value);
  RooFormulaVar deltaM("deltaM","deltaM","(@0 - @1)",RooArgList(d0pi, d0_mass));
  //Separated datasets
  RooDataSet *dataxy_d0 = new RooDataSet("data_d0","data_d0",tree,RooArgSet(log_min_significance,d0pi,pisoft),"pisoft==1");
  RooDataSet *dataxy_d0bar = new RooDataSet("data_d0bar","data_d0bar",tree,RooArgSet(log_min_significance,d0pi,pisoft),"pisoft==-1");  
  
  /*-----------------------S I G N A L----------------------*/
  //deltaM
  RooRealVar mean_sig("#mu_{sig}", "mean",1.45420e-01);
  RooRealVar A_mean_sig("A_mean_sig","Asymmetry in mean",0);
  RooFormulaVar mean_sig_d0("mean_sig_d0","mean","(@0*(1 + @1))",RooArgList(mean_sig, A_mean_sig));
  RooFormulaVar mean_sig_d0bar("mean_sig_d0bar","mean","(@0*(1- @1))",RooArgList(mean_sig, A_mean_sig));

  RooRealVar lambda_sig("#lambda_{sig}", "sigma", 1.58131e-04);
  RooRealVar A_lambda_sig("A_lambda_sig","Asymmetry in lambda",0);
  RooFormulaVar lambda_sig_d0("lambda_sig_d0","lambda","(@0*(1 + @1))",RooArgList(lambda_sig, A_lambda_sig));
  RooFormulaVar lambda_sig_d0bar("lambda_sig_d0bar","lambda","(@0*(1- @1))",RooArgList(lambda_sig, A_lambda_sig));

  RooRealVar delta_sig("#delta_{sig}", "delta", -5.59429e-02);
  RooRealVar A_delta_sig("A_delta_sig","Asymmetry in delta",0);
  RooFormulaVar delta_sig_d0("delta_sig_d0","delta","(@0*(1 + @1))",RooArgList(delta_sig, A_delta_sig));
  RooFormulaVar delta_sig_d0bar("delta_sig_d0bar","delta","(@0*(1- @1))",RooArgList(delta_sig, A_delta_sig));

  RooRealVar gamma_sig("#gamma_{sig}", "gamma", 7.75617e-01);
  RooRealVar A_gamma_sig("A_gamma_sig","Asymmetry in gamma",0);
  RooFormulaVar gamma_sig_d0("gamma_sig_d0","gamma","(@0*(1 + @1))",RooArgList(gamma_sig, A_gamma_sig));
  RooFormulaVar gamma_sig_d0bar("gamma_sig_d0bar","gamma","(@0*(1- @1))",RooArgList(gamma_sig, A_gamma_sig));

  RooJohnson deltam_sig_d0("deltam_sig_d0", "signal component_d0", deltaM, mean_sig_d0, lambda_sig_d0, delta_sig_d0, gamma_sig_d0);
  RooJohnson deltam_sig_d0bar("deltam_sig_d0bar", "signal component_d0bar", deltaM, mean_sig_d0bar, lambda_sig_d0bar, delta_sig_d0bar, gamma_sig_d0bar);

  //log_min_significance
  RooRealVar gamma_mean_sig("#mu_{sig_gamma}", "mean", 7.09566e+00);
  RooRealVar A_gamma_mean_sig("A_gamma_mean_sig","Asymmetry in gamma_mean_sig",0);
  RooFormulaVar gamma_mean_sig_d0("gamma_mean_sig_d0","gamma_mean","(@0*(1 + @1))",RooArgList(gamma_mean_sig, A_gamma_mean_sig));
  RooFormulaVar gamma_mean_sig_d0bar("gamma_mean_sig_d0bar","gamma_mean","(@0*(1- @1))",RooArgList(gamma_mean_sig, A_gamma_mean_sig));

  RooRealVar gamma_sigma_sig("#lambda_{sig_gamma}", "sigma",3.55381e-02);
  RooRealVar A_gamma_sigma_sig("A_gamma_sigma_sig","Asymmetry in gamma_sigma_sig",0);
  RooFormulaVar gamma_sigma_sig_d0("gamma_sigma_sig_d0","gamma_sigma","(@0*(1 + @1))",RooArgList(gamma_sigma_sig, A_gamma_sigma_sig));
  RooFormulaVar gamma_sigma_sig_d0bar("gamma_sigma_sig_d0bar","gamma_sigma","(@0*(1- @1))",RooArgList(gamma_sigma_sig, A_gamma_sigma_sig));

  RooRealVar gamma_delta_sig("#delta_{sig_gamma}", "delta", 9.46867e+00);
  RooRealVar A_gamma_delta_sig("A_gamma_delta_sig","Asymmetry in gamma_delta_sig",0);
  RooFormulaVar gamma_delta_sig_d0("gamma_delta_sig_d0","gamma_delta","(@0*(1 + @1))",RooArgList(gamma_delta_sig, A_gamma_delta_sig));
  RooFormulaVar gamma_delta_sig_d0bar("gamma_delta_sig_d0bar","gamma_delta","(@0*(1- @1))",RooArgList(gamma_delta_sig, A_gamma_delta_sig));

  RooRealVar gamma_gamma_sig("#gamma_{sig_gamma}", "gamma", 2.00211e+00);
  RooRealVar A_gamma_gamma_sig("A_gamma_gamma_sig","Asymmetry in gamma_gamma_sig",0);
  RooFormulaVar gamma_gamma_sig_d0("gamma_gamma_sig_d0","gamma_gamma","(@0*(1 + @1))",RooArgList(gamma_gamma_sig, A_gamma_gamma_sig));
  RooFormulaVar gamma_gamma_sig_d0bar("gamma_gamma_sig_d0bar","gamma_gamma","(@0*(1- @1))",RooArgList(gamma_gamma_sig, A_gamma_gamma_sig));

  RooJohnson gammapdf_sig_d0("gammapdf_sig_d0", "signal component_d0", log_min_significance, gamma_mean_sig_d0, gamma_sigma_sig_d0, gamma_delta_sig, gamma_gamma_sig_d0);
  RooJohnson gammapdf_sig_d0bar("gammapdf_sig_d0bar", "signal component_d0bar", log_min_significance, gamma_mean_sig_d0bar, gamma_sigma_sig_d0bar, gamma_delta_sig_d0bar, gamma_gamma_sig_d0bar);
  
  //total signal component
  RooProdPdf signal_peak_d0("signal_peak_d0","signal_peak_d0", RooArgSet(deltam_sig_d0,gammapdf_sig_d0));
  RooProdPdf signal_peak_d0bar("signal_peak_d0bar","signal_peak_d0bar", RooArgSet(deltam_sig_d0bar,gammapdf_sig_d0bar));
  
  // /*-----------------------P E A K I N G     B A C K G R O U N D     (KS Pi Pi)-------------------------*/

  //deltaM
  RooRealVar f_G_kspipi("f_{G_kspipi}", "",2.33590e-01);
  RooRealVar A_f_G_kspipi("A_f_G_kspipi","Asymmetry in f_G_kspipi",0);
  RooFormulaVar f_G_kspipi_d0("f_G_kspipi_d0","f_G_kspipi","(@0*(1 + @1))",RooArgList(f_G_kspipi, A_f_G_kspipi));
  RooFormulaVar f_G_kspipi_d0bar("f_G_kspipi_d0bar","f_G_kspipi","(@0*(1- @1))",RooArgList(f_G_kspipi, A_f_G_kspipi));

  RooRealVar mean_G_kspipi("#mu_{G_kspipi}", "mean",1.45423e-01);
  RooRealVar A_mean_G_kspipi("A_mean_G_kspipi","Asymmetry in mean_G_kspipi",0);
  RooFormulaVar mean_G_kspipi_d0("mean_G_kspipi_d0","mean_G_kspipi","(@0*(1 + @1))",RooArgList(mean_G_kspipi, A_mean_G_kspipi));
  RooFormulaVar mean_G_kspipi_d0bar("mean_G_kspipi_d0bar","mean_G_kspipi","(@0*(1- @1))",RooArgList(mean_G_kspipi, A_mean_G_kspipi));

  RooRealVar sigma_G_kspipi("#sigma_{G_kspipi}", "sigma",1.84760e-04);
  RooRealVar A_sigma_G_kspipi("A_sigma_G_kspipi","Asymmetry in sigma_G_kspipi",0);
  RooFormulaVar sigma_G_kspipi_d0("sigma_G_kspipi_d0","sigma_G_kspipi","(@0*(1 + @1))",RooArgList(sigma_G_kspipi, A_sigma_G_kspipi));
  RooFormulaVar sigma_G_kspipi_d0bar("sigma_G_kspipi_d0bar","sigma_G_kspipi","(@0*(1- @1))",RooArgList(sigma_G_kspipi, A_sigma_G_kspipi));
  
  RooGaussian gauss_kspipi_d0("gauss_kspipi_d0", "gauss component_d0", deltaM, mean_G_kspipi_d0, sigma_G_kspipi_d0);
  RooGaussian gauss_kspipi_d0bar("gauss_kspipi_d0bar", "gauss component_d0bar", deltaM, mean_G_kspipi_d0bar, sigma_G_kspipi_d0bar);

  RooRealVar lambda_J_kspipi("#lambda_{J_kspipi}", "lambda", 1.40786e-04);
  RooRealVar A_lambda_J_kspipi("A_lambda_J_kspipi","Asymmetry in lambda_J_kspipi",0);
  RooFormulaVar lambda_J_kspipi_d0("lambda_J_kspipi_d0","lambda_J_kspipi","(@0*(1 + @1))",RooArgList(lambda_J_kspipi, A_lambda_J_kspipi));
  RooFormulaVar lambda_J_kspipi_d0bar("lambda_J_kspipi_d0bar","lambda_J_kspipi","(@0*(1- @1))",RooArgList(lambda_J_kspipi, A_lambda_J_kspipi));

  RooRealVar delta_J_kspipi("#delta_{J_kspipi}", "delta",-4.03550e-02);
  RooRealVar A_delta_J_kspipi("A_delta_J_kspipi","Asymmetry in delta_J_kspipi",0);
  RooFormulaVar delta_J_kspipi_d0("delta_J_kspipi_d0","delta_J_kspipi","(@0*(1 + @1))",RooArgList(delta_J_kspipi, A_delta_J_kspipi));
  RooFormulaVar delta_J_kspipi_d0bar("delta_J_kspipi_d0bar","delta_J_kspipi","(@0*(1- @1))",RooArgList(delta_J_kspipi, A_delta_J_kspipi));

  RooRealVar gamma_J_kspipi("#gamma_{J_kspipi}", "gamma",6.83756e-01);
  RooRealVar A_gamma_J_kspipi("A_gamma_J_kspipi","Asymmetry in gamma_J_kspipi",0);
  RooFormulaVar gamma_J_kspipi_d0("gamma_J_kspipi_d0","gamma_J_kspipi","(@0*(1 + @1))",RooArgList(gamma_J_kspipi, A_gamma_J_kspipi));
  RooFormulaVar gamma_J_kspipi_d0bar("gamma_J_kspipi_d0bar","gamma_J_kspipi","(@0*(1- @1))",RooArgList(gamma_J_kspipi, A_gamma_J_kspipi));

  RooJohnson johnson_kspipi_d0("johnson_kspipi_d0", "johnson component_d0", deltaM, mean_G_kspipi_d0, lambda_J_kspipi_d0, delta_J_kspipi_d0, gamma_J_kspipi_d0);
  RooJohnson johnson_kspipi_d0bar("johnson_kspipi_d0bar", "johnson component_d0bar", deltaM, mean_G_kspipi_d0bar, lambda_J_kspipi_d0bar, delta_J_kspipi_d0bar, gamma_J_kspipi_d0bar);

  RooAddPdf deltam_kspipi_d0("deltam_kspipi_d0" ,"gauss_kspipi_d0 + johnson_kspipi_d0",gauss_kspipi_d0, johnson_kspipi_d0,f_G_kspipi_d0);
  RooAddPdf deltam_kspipi_d0bar("deltam_kspipi_d0bar" ,"gauss_kspipi_d0bar + johnson_kspipi_d0bar",gauss_kspipi_d0bar, johnson_kspipi_d0bar,f_G_kspipi_d0bar);
  
  //log_min_significance
  RooRealVar gamma_mean_kspipi("#mu_{kspipi_gamma}", "mean",2.24177e+00);
  RooRealVar A_gamma_mean_kspipi("A_gamma_mean_kspipi","Asymmetry in gamma_mean_kspipi",0);
  RooFormulaVar gamma_mean_kspipi_d0("gamma_mean_kspipi_d0","gamma_mean","(@0*(1 + @1))",RooArgList(gamma_mean_kspipi, A_gamma_mean_kspipi));
  RooFormulaVar gamma_mean_kspipi_d0bar("gamma_mean_kspipi_d0bar","gamma_mean","(@0*(1- @1))",RooArgList(gamma_mean_kspipi, A_gamma_mean_kspipi));

  RooRealVar gamma_sigma_kspipi("#lambda_{kspipi_gamma}", "sigma",1.38958e+00);
  RooRealVar A_gamma_sigma_kspipi("A_gamma_sigma_kspipi","Asymmetry in gamma_sigma_kspipi",0);
  RooFormulaVar gamma_sigma_kspipi_d0("gamma_sigma_kspipi_d0","gamma_sigma","(@0*(1 + @1))",RooArgList(gamma_sigma_kspipi, A_gamma_sigma_kspipi));
  RooFormulaVar gamma_sigma_kspipi_d0bar("gamma_sigma_kspipi_d0bar","gamma_sigma","(@0*(1- @1))",RooArgList(gamma_sigma_kspipi, A_gamma_sigma_kspipi));

  RooRealVar gamma_delta_kspipi("#delta_{kspipi_gamma}", "delta",1.78959e+00);
  RooRealVar A_gamma_delta_kspipi("A_gamma_delta_kspipi","Asymmetry in gamma_delta_kspipi",0);
  RooFormulaVar gamma_delta_kspipi_d0("gamma_delta_kspipi_d0","gamma_delta","(@0*(1 + @1))",RooArgList(gamma_delta_kspipi, A_gamma_delta_kspipi));
  RooFormulaVar gamma_delta_kspipi_d0bar("gamma_delta_kspipi_d0bar","gamma_delta","(@0*(1- @1))",RooArgList(gamma_delta_kspipi, A_gamma_delta_kspipi));

  RooRealVar gamma_gamma_kspipi("#gamma_{kspipi_gamma}", "gamma", 1.92639e+00);
  RooRealVar A_gamma_gamma_kspipi("A_gamma_gamma_kspipi","Asymmetry in gamma_gamma_kspipi",0);
  RooFormulaVar gamma_gamma_kspipi_d0("gamma_gamma_kspipi_d0","gamma_gamma","(@0*(1 + @1))",RooArgList(gamma_gamma_kspipi, A_gamma_gamma_kspipi));
  RooFormulaVar gamma_gamma_kspipi_d0bar("gamma_gamma_kspipi_d0bar","gamma_gamma","(@0*(1- @1))",RooArgList(gamma_gamma_kspipi, A_gamma_gamma_kspipi));

  RooJohnson gamma_kspipi_d0("gamma_kspipi_d0", "signal component_d0", log_min_significance, gamma_mean_kspipi_d0, gamma_sigma_kspipi_d0, gamma_delta_kspipi, gamma_gamma_kspipi_d0);
  RooJohnson gamma_kspipi_d0bar("gamma_kspipi_d0bar", "signal component_d0bar", log_min_significance, gamma_mean_kspipi_d0bar, gamma_sigma_kspipi_d0bar, gamma_delta_kspipi_d0bar, gamma_gamma_kspipi_d0bar);   
  
  //kspipi component (peaking background)
  RooProdPdf kspipi_peak_d0("kspipi_peak_d0", "kspipi_peak_d0", deltam_kspipi_d0, gamma_kspipi_d0);
  RooProdPdf kspipi_peak_d0bar("kspipi_peak_d0bar", "kspipi_peak_d0bar", deltam_kspipi_d0bar, gamma_kspipi_d0bar);

  
  /*----------------------- N O N     P E A K I N G     B A C K G R O U N D-------------------------*/
  //deltaM
  // RooRealVar mpi_non_peaking("mpi_non_peaking", "nominal pion mass",2.00441);
  RooRealVar mpi_non_peaking("mpi_non_peaking", "nominal pion mass", 0.13957039);
  RooRealVar alpha_non_peaking("#alpha_{non_pkng_bkg}", "alpha", -2.30301e+01, -1e3, 1e3);
  RooRealVar A_alpha_non_peaking("A_alpha_non_peaking","Asymmetry in alpha_non_peaking",0);
  RooFormulaVar alpha_non_peaking_d0("alpha_non_peaking_d0","alpha_non_peaking_d0","(@0*(1 + @1))",RooArgList(alpha_non_peaking, A_alpha_non_peaking));
  RooFormulaVar alpha_non_peaking_d0bar("alpha_non_peaking_d0bar","alpha_non_peaking_d0bar","(@0*(1- @1))",RooArgList(alpha_non_peaking, A_alpha_non_peaking));
  RooRealVar beta_non_peaking("#beta_{non_peaking}", "beta", 0);
  
  myRooPolBG deltam_non_peaking_d0("deltam_non_peaking_d0","deltam_non_peaking_d0", deltaM, mpi_non_peaking, alpha_non_peaking_d0, beta_non_peaking);
  myRooPolBG deltam_non_peaking_d0bar("deltam_non_peaking_d0bar","deltam_non_peaking_d0bar", deltaM, mpi_non_peaking, alpha_non_peaking_d0bar, beta_non_peaking);


  //log_min_significance
  RooRealVar f_L_non_peaking("fL_{non_peaking}", "frac",7.52708e-01);
  RooRealVar A_fL_non_peaking(" A_fL_non_peaking","Asymmetry in gamma_mean_non_peaking",0);
  RooFormulaVar f_L_non_peaking_d0("f_L_non_peaking_d0","gamma_mean","(@0*(1 + @1))",RooArgList(f_L_non_peaking, A_fL_non_peaking));
  RooFormulaVar f_L_non_peaking_d0bar("f_L_non_peaking_d0","gamma_mean","(@0*(1- @1))",RooArgList(f_L_non_peaking, A_fL_non_peaking));

  RooRealVar gamma_meanL("#mu_{L}", "mean",4.40036e+00);
  RooRealVar A_gamma_meanL("A_gamma_meanL","Asymmetry in gamma_mean_non_peaking",0);
  RooFormulaVar gamma_meanL_d0("ggamma_meanL_d0","gamma_mean","(@0*(1 + @1))",RooArgList(gamma_meanL, A_gamma_meanL));
  RooFormulaVar gamma_meanL_d0bar("gamma_meanL_d0bar","gamma_mean","(@0*(1- @1))",RooArgList(gamma_meanL, A_gamma_meanL));

  RooRealVar gamma_sigmaL("lambda_{L}", "sigma",1.51278e+00);
  RooRealVar A_gamma_sigmaL("A_gamma_sigmaL","Asymmetry in gamma_mean_non_peaking",0);
  RooFormulaVar gamma_sigmaL_d0("gamma_sigmaL_d0","gamma_mean","(@0*(1 + @1))",RooArgList(gamma_sigmaL, A_gamma_sigmaL));
  RooFormulaVar gamma_sigmaL_d0bar("gamma_sigmaL_d0bar","gamma_mean","(@0*(1- @1))",RooArgList(gamma_sigmaL, A_gamma_sigmaL));
  
  RooRealVar gamma_deltaL("delta_{L}", "mean",6.69466e+00);
  RooRealVar A_gamma_deltaL("A_gamma_deltaL","Asymmetry in gamma_mean_non_peaking",0);
  RooFormulaVar gamma_deltaL_d0("gamma_deltaL_d0","gamma_mean","(@0*(1 + @1))",RooArgList(gamma_deltaL, A_gamma_deltaL));
  RooFormulaVar gamma_deltaL_d0bar("gamma_deltaL_d0bar","gamma_mean","(@0*(1- @1))",RooArgList(gamma_deltaL, A_gamma_deltaL));
  
  RooRealVar gamma_gammaL("gamma_{L}", "delta",3.83292e+00);
  RooRealVar A_gamma_gammaL("A_gamma_gammaL","Asymmetry in gamma_mean_non_peaking",0);
  RooFormulaVar gamma_gammaL_d0("gamma_gammaL_d0","gamma_mean","(@0*(1 + @1))",RooArgList(gamma_gammaL, A_gamma_gammaL));
  RooFormulaVar gamma_gammaL_d0bar("gamma_gammaL_d0bar","gamma_mean","(@0*(1- @1))",RooArgList(gamma_gammaL, A_gamma_gammaL));

  RooJohnson gammaL_non_peaking_d0("gammaL_non_peaking_d0", "signal component_d0", log_min_significance, gamma_meanL_d0, gamma_sigmaL_d0, gamma_deltaL_d0, gamma_gammaL_d0);
  RooJohnson gammaL_non_peaking_d0bar("gammaL_non_peaking_d0bar", "signal component_d0", log_min_significance, gamma_meanL_d0bar, gamma_sigmaL_d0bar, gamma_deltaL_d0bar, gamma_gammaL_d0bar);
  
  ///////////////////////// 
  RooRealVar gamma_meanR("#mu_{R}", "mean",6.89119e+00);
  RooRealVar A_gamma_meanR("A_gamma_meanR","Asymmetry in gamma_mean_non_peaking",0);
  RooFormulaVar gamma_meanR_d0("ggamma_meanR_d0","gamma_mean","(@0*(1 + @1))",RooArgList(gamma_meanR, A_gamma_meanR));
  RooFormulaVar gamma_meanR_d0bar("gamma_meanR_d0bar","gamma_mean","(@0*(1- @1))",RooArgList(gamma_meanR, A_gamma_meanR));

  RooRealVar gamma_sigmaR("lambda_{R}", "sigma",5.29095e-03);
  RooRealVar A_gamma_sigmaR("A_gamma_sigmaR","Asymmetry in gamma_mean_non_peaking",0);
  RooFormulaVar gamma_sigmaR_d0("gamma_sigmaR_d0","gamma_mean","(@0*(1 + @1))",RooArgList(gamma_sigmaR, A_gamma_sigmaR));
  RooFormulaVar gamma_sigmaR_d0bar("gamma_sigmaR_d0bar","gamma_mean","(@0*(1- @1))",RooArgList(gamma_sigmaR, A_gamma_sigmaR));

  RooRealVar gamma_deltaR("delta_{R}", "delta",9.93847e+00);
  RooRealVar A_gamma_deltaR("A_gamma_deltaR","Asymmetry in gamma_mean_non_peaking",0);
  RooFormulaVar gamma_deltaR_d0("gamma_deltaR_d0","gamma_mean","(@0*(1 + @1))",RooArgList(gamma_deltaR, A_gamma_deltaR));
  RooFormulaVar gamma_deltaR_d0bar("gamma_deltaR_d0bar","gamma_mean","(@0*(1- @1))",RooArgList(gamma_deltaR, A_gamma_deltaR));

  RooRealVar gamma_gammaR("gamma_{R}", "gamma",1.44091e+00);
  RooRealVar A_gamma_gammaR("A_gamma_gammaR","Asymmetry in gamma_mean_non_peaking",0);
  RooFormulaVar gamma_gammaR_d0("gamma_gammaR_d0","gamma_mean","(@0*(1 + @1))",RooArgList(gamma_gammaR, A_gamma_gammaR));
  RooFormulaVar gamma_gammaR_d0bar("gamma_gammaR_d0bar","gamma_mean","(@0*(1- @1))",RooArgList(gamma_gammaR, A_gamma_gammaR));
 
  RooJohnson gammaR_non_peaking_d0("gammaR_non_peaking_d0", "signal component_d0", log_min_significance, gamma_meanR_d0, gamma_sigmaR_d0, gamma_deltaR_d0, gamma_gammaR_d0);
  RooJohnson gammaR_non_peaking_d0bar("gammaR_non_peaking_d0bar", "signal component_d0", log_min_significance, gamma_meanR_d0bar, gamma_sigmaR_d0bar, gamma_deltaR_d0bar, gamma_gammaR_d0bar);
  
  RooAddPdf gamma_non_peaking_d0("gamma_non_peaking_d0", "gammaL_non_peaking_d0 + gammaR_non_peaking_d0",gammaL_non_peaking_d0,gammaR_non_peaking_d0,f_L_non_peaking_d0);
  RooAddPdf gamma_non_peaking_d0bar("gamma_non_peaking_d0bar", "gammaL_non_peaking_d0bar + gammaR_non_peaking_d0bar",gammaL_non_peaking_d0bar,gammaR_non_peaking_d0bar,f_L_non_peaking_d0bar); 

  //non_peaking component
  RooProdPdf non_peaking_d0("non_peaking_d0", "non_peaking_d0", deltam_non_peaking_d0, gamma_non_peaking_d0);
  RooProdPdf non_peaking_d0bar("non_peaking_d0bar", "non_peaking_d0bar", deltam_non_peaking_d0bar, gamma_non_peaking_d0bar); 
  
  // /*------------------------Y I E L D S  F O R  A L L  C O M P O N E N T S----------------------------*/
  RooRealVar signal_yield("N_{sig}","signal yield",-3.37467e+01,1e0,1e4);
  RooRealVar kspipi_yield("N_{pkng_bkg}","kspipi yield",6.26614e+03,1e0,1e5);
  RooRealVar nonpeaking_yield("N_{non_pkng_bkg}","nonpeaking yield", 4.92391e+03,1e0,1e5);

   //For Blinding signal asymmetry
  // TRandom3 *r=new TRandom3(1994);
  // Double_t shift = r->Uniform(-0.1,0.1); 
  // cout << "random = "<< shift << endl; 
  // RooRealVar rnd("rnd","",shift,"");

  RooRealVar Araw_sig("A_{sig}","Araw",0,-1,1);
  //RooFormulaVar Araw_sig_blind("A_{sig_blind}","(@0 + @1)",RooArgList(rnd,Araw_sig));
  RooRealVar Araw_kspipi("A_{pkng_bkg}","Araw",0,-1,1);
  RooRealVar Araw_nonpeaking("A_{non_pkng_bkg}","Araw",0,-1,1);

  //Yields for D0 sample: ND0 = N(1+A)/2
  RooFormulaVar signal_yield_d0("ND0_{signal}","(@0*(1 + @1))/2",RooArgList(signal_yield,Araw_sig));
  RooFormulaVar kspipi_yield_d0("ND0_{kspipi}","(@0*(1 + @1))/2",RooArgList(kspipi_yield,Araw_kspipi));
  RooFormulaVar nonpeaking_yield_d0("ND0_{nonpeaking}","(@0*(1 + @1))/2",RooArgList(nonpeaking_yield,Araw_nonpeaking));

  //Yields for D0 sample: ND0 = N(1-A)/2
  RooFormulaVar signal_yield_d0bar("ND0bar_{signal}","(@0*(1 - @1))/2",RooArgList(signal_yield,Araw_sig));
  RooFormulaVar kspipi_yield_d0bar("ND0bar_{kspipi}","(@0*(1 - @1))/2",RooArgList(kspipi_yield,Araw_kspipi));
  RooFormulaVar nonpeaking_yield_d0bar("ND0bar_{nonpeaking}","(@0*(1 - @1))/2",RooArgList(nonpeaking_yield,Araw_nonpeaking));

  
  /*------------------------C O M P L E T E  M O D E L----------------------------*/
  RooAddPdf model_d0("model_d0","",RooArgList(signal_peak_d0,kspipi_peak_d0,non_peaking_d0), RooArgList(signal_yield_d0,kspipi_yield_d0,nonpeaking_yield_d0));
  RooAddPdf model_d0bar("model_d0bar","",RooArgList(signal_peak_d0bar,kspipi_peak_d0bar,non_peaking_d0bar), RooArgList(signal_yield_d0bar,kspipi_yield_d0bar,nonpeaking_yield_d0bar));

  RooAddPdf signal_peak("signal_peak", "signal_peak", RooArgList(signal_peak_d0, signal_peak_d0bar),RooArgList(signal_yield_d0, signal_yield_d0bar));
  RooAddPdf kspipi_peak("kspipi_peak", "kspipi_peak", RooArgList(kspipi_peak_d0, kspipi_peak_d0bar),RooArgList(kspipi_yield_d0, kspipi_yield_d0bar));
  RooAddPdf non_peaking("non_peaking", "non_peaking", RooArgList(non_peaking_d0, non_peaking_d0bar),RooArgList(nonpeaking_yield_d0, nonpeaking_yield_d0bar));
  RooAddPdf model("model","model", RooArgList(signal_peak, kspipi_peak, non_peaking), RooArgList(signal_yield, kspipi_yield, nonpeaking_yield));

  //Define category to distinguish d0 and d0bar samples events
  RooCategory sample("sample","sample") ;
  sample.defineType("D0") ;
  sample.defineType("D0bar") ;

  //Construct combined dataset
  RooDataSet combData("combData","combined data",RooArgSet(log_min_significance,d0pi),Index(sample),Import("D0",*dataxy_d0),Import("D0bar",*dataxy_d0bar));
  
  //Construct a simultaneous pdf using category sample as index
  RooSimultaneous simPdf("simPdf","simultaneous pdf",sample) ;
  simPdf.addPdf(model_d0,"D0") ;
  simPdf.addPdf(model_d0bar,"D0bar");

  //Perform the simultaneous fit
  simPdf.fitTo(combData,Strategy(2));


  RooPlot *deltaM_d0 = d0pi.frame(Title(" "));
  combData.plotOn(deltaM_d0, Name("DeltaM"));
  simPdf.plotOn(deltaM_d0, ProjWData(sample,combData));
  simPdf.paramOn(deltaM_d0,dataxy_d0);
  deltaM_d0->SetXTitle("#it{#Deltam} [GeV/c^{2}]");
  deltaM_d0->SetYTitle("Candidates per 0.2 MeV/#it{c}^{2}");
  deltaM_d0->GetYaxis()->SetTitleOffset(1.5);
  deltaM_d0->SetLabelOffset(99);
  deltaM_d0->SetTitleOffset(99);

  // RooHist* hpullx_d0_deltaM = deltaM_d0->pullHist();
  // hpullx_d0_deltaM->SetFillStyle(1001);
  // hpullx_d0_deltaM->SetFillColor(1);
  // for(int i=0;i<hpullx_d0_deltaM->GetN();++i) hpullx_d0_deltaM->SetPointError(i,0.0,0.0,0.0,0.0);
  // RooPlot* pullplotx_d0_deltaM = d0pi.frame(Title(" ")) ;
  // pullplotx_d0_deltaM->addPlotable(hpullx_d0_deltaM,"B");
  // pullplotx_d0_deltaM->SetXTitle("#it{m(D^{0}#pi^{+}_{s})} [GeV/c^{2}]");
  // pullplotx_d0_deltaM->SetMinimum(-4.);
  // pullplotx_d0_deltaM->SetMaximum(4.);
  // pullplotx_d0_deltaM->GetXaxis()->SetLabelSize(0.1);
  // pullplotx_d0_deltaM->GetXaxis()->SetTitleSize(0.1);
  // pullplotx_d0_deltaM->GetYaxis()->SetLabelSize(0.07);
  
  TCanvas* c1 = new TCanvas("c1", "c1",700, 700);
  Double_t xlow, ylow, xup, yup;
  c1->GetPad(0)->GetPadPar(xlow,ylow,xup,yup);
  c1->Divide(1,2);
  TVirtualPad *upPad1 = c1->GetPad(1);
  upPad1->SetPad(xlow,ylow+0.25*(yup-ylow),xup,yup);
  TVirtualPad *dwPad1 = c1->GetPad(2);
  dwPad1->SetPad(xlow,ylow,xup,ylow+0.25*(yup-ylow));
  c1->Update();
  c1->cd(1);
  model.plotOn(deltaM_d0,Components(RooArgSet(model)),Name("model"));
  model.plotOn(deltaM_d0,Components(RooArgSet(signal_peak)),LineStyle(kDashed),LineColor(kRed),Name("signal_peak"));
  model.plotOn(deltaM_d0,Components(RooArgSet(kspipi_peak)),LineStyle(kDashed),LineColor(kGreen),Name("kspipi_peak"));
  model.plotOn(deltaM_d0,Components(RooArgSet(non_peaking)),LineStyle(kDashed),LineColor(kBlack),Name("non_peaking"));
  
  deltaM_d0->Draw();

  TLatex *logo1 = new TLatex();
  logo1->SetNDC();
  // logo1->DrawLatex(0.55,0.87, "#bf{#it{Belle II}}");
  // logo1->DrawLatex(0.55,0.80, "#it{#scale[0.6]{#int} L dt} = 400 fb^{#minus1}");
  // logo1->DrawLatex(0.55,0.45,"#it{D}^{0}#rightarrow#it{K_{s}^{0}K_{s}^{0}}");

  TLegend *leg_deltaM = new TLegend(0.54,0.32,0.84,0.52);
  leg_deltaM->SetBorderSize(0);  // no border
  leg_deltaM->AddEntry(deltaM_d0->findObject("DeltaM"),"Data","pe");
  leg_deltaM->AddEntry(deltaM_d0->findObject("model"),"Total Fit","l");
  leg_deltaM->AddEntry(deltaM_d0->findObject("signal_peak"),"D^{0}#rightarrow K^{0}_{s}K^{0}_{s}","l");
  leg_deltaM->AddEntry(deltaM_d0->findObject("kspipi_peak"),"D^{0}#rightarrow K^{0}_{s}#pi^{+}#pi^{-}","l");
  leg_deltaM->AddEntry(deltaM_d0->findObject("non_peaking"),"Other Backgrounds","l");
  leg_deltaM->Draw();
  c1->Update();


 
  // c1->cd(2);
  // TFile *f_asym=new TFile("/home/sanjeeda/Desktop/thesis_new/analysis/iriomote/belle2/ksks/asymmetry_projections/belle2_data_ksks_asym_proj.root"); 
  // TH1F *h1;
  // TH1F *h2;
  // TH1  *h_asym;
  // h1 = (TH1F*)f_asym->Get("histogram_deltam_d0__deltaM");
  // h2 = (TH1F*)f_asym->Get("histogram_deltam_d0bar__deltaM");

  // h_asym =h1->GetAsymmetry(h2);
  // // // pullplotx_d0_deltaM->Draw();
  // h_asym->Draw();
  // h_asym->GetYaxis()->SetRangeUser(-0.5, 0.5); 
  c1->Update();

  // Plot simultaneous pdf for gamma.
  RooPlot *log_min_significance_d0 = log_min_significance.frame(Title(" "));
  combData.plotOn(log_min_significance_d0, Name("gamma"));
  simPdf.plotOn(log_min_significance_d0, ProjWData(sample,combData));
  log_min_significance_d0->SetXTitle("#gamma");
  log_min_significance_d0->SetYTitle("Candidates per 0.13");
  log_min_significance_d0->GetYaxis()->SetTitleOffset(1.5);
  log_min_significance_d0->SetLabelOffset(99);
  log_min_significance_d0->SetTitleOffset(99);
 
  RooHist* hpully_d0_log_min_significance = log_min_significance_d0->pullHist();
  hpully_d0_log_min_significance->SetFillStyle(1001);
  hpully_d0_log_min_significance->SetFillColor(1);
  for(int i=0;i<hpully_d0_log_min_significance->GetN();++i) hpully_d0_log_min_significance->SetPointError(i,0.0,0.0,0.0,0.0);
  RooPlot* pullploty_d0_log_min_significance = log_min_significance.frame(Title(" ")) ;
  pullploty_d0_log_min_significance->addPlotable(hpully_d0_log_min_significance,"B");
  pullploty_d0_log_min_significance->SetXTitle("#gamma");
  pullploty_d0_log_min_significance->SetMinimum(-4.);
  pullploty_d0_log_min_significance->SetMaximum(4.);
  pullploty_d0_log_min_significance->GetXaxis()->SetLabelSize(0.1);
  pullploty_d0_log_min_significance->GetXaxis()->SetTitleSize(0.1);
  pullploty_d0_log_min_significance->GetYaxis()->SetLabelSize(0.07);
  
  TCanvas* c3 = new TCanvas("c3", "c3",700, 700);
  c3->GetPad(0)->GetPadPar(xlow,ylow,xup,yup);
  c3->Divide(1,2);
  TVirtualPad *upPad3 = c3->GetPad(1);
  upPad3->SetPad(xlow,ylow+0.25*(yup-ylow),xup,yup);
  TVirtualPad *dwPad3 = c3->GetPad(2);
  dwPad3->SetPad(xlow,ylow,xup,ylow+0.25*(yup-ylow));
  c3->Update();
  c3->cd(1);

  model.plotOn(log_min_significance_d0,Components(RooArgSet(model)),Name("model"));
  model.plotOn(log_min_significance_d0,Components(RooArgSet(signal_peak)),LineStyle(kDashed),LineColor(kRed),Name("signal_peak_gamma"));
  model.plotOn(log_min_significance_d0,Components(RooArgSet(kspipi_peak)),LineStyle(kDashed),LineColor(kGreen),Name("kspipi_peak_gamma"));
  model.plotOn(log_min_significance_d0,Components(RooArgSet(non_peaking)),LineStyle(kDashed),LineColor(kBlack),Name("non_peaking_gamma"));
  
  log_min_significance_d0->Draw();

  TLatex *logo2 = new TLatex();
  logo2->SetNDC();
  // logo2->DrawLatex(0.55,0.87, "#bf{#it{Belle II}}");
  // logo2->DrawLatex(0.55,0.80, "#it{#scale[0.6]{#int} L dt} = 400 fb^{#minus1}");
  // logo2->DrawLatex(0.55,0.45,"#it{D}^{0}#rightarrow#it{K_{s}^{0}K_{s}^{0}}");
  // logo2->DrawLatex(0.55,0.37,"#it{N} = 5852 #pm 83");

  TLegend *leg_log_min_significance = new TLegend(0.54,0.32,0.84,0.52);
  leg_log_min_significance->SetBorderSize(0);  // no border
  leg_log_min_significance->AddEntry(log_min_significance_d0->findObject("gamma"),"Data","pe");
  leg_log_min_significance->AddEntry(log_min_significance_d0->findObject("model"),"Total Fit","l");
  leg_log_min_significance->AddEntry(log_min_significance_d0->findObject("signal_peak_gamma"),"D^{0}#rightarrow K^{0}_{s}K^{0}_{s}","l");
  leg_log_min_significance->AddEntry(log_min_significance_d0->findObject("kspipi_peak_gamma"),"D^{0}#rightarrow K^{0}_{s}#pi^{+}#pi^{-}","l");
  leg_log_min_significance->AddEntry(log_min_significance_d0->findObject("non_peaking_gamma"),"Other Backgrounds","l");
  leg_log_min_significance->Draw();

  c3->cd(2);
  pullploty_d0_log_min_significance->Draw();
  c3->Update();  


  c1->cd(2);
  TFile *f_asym=new TFile("belle2_data_ksks_asym_proj.root"); 
  TH1F *h1;
  TH1F *h2;
  TH1  *h_asym;
  h1 = (TH1F*)f_asym->Get("histogram_deltam_d0__deltaM");
  h2 = (TH1F*)f_asym->Get("histogram_deltam_d0bar__deltaM");

  h_asym =h1->GetAsymmetry(h2);
  // // pullplotx_d0_deltaM->Draw();
  h_asym->Draw();
  c1->Update();
  

  
}
